// VRLib.cpp : Defines the entry point for the DLL application.
//

#include "stdafx.h"
#include <stdlib.h>
#include "VRLib.h"

BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}

// This is an example of an exported variable
VRLIB_API int nVRLib=0;

// This is an example of an exported function.
VRLIB_API int fnVRLib(void)
{
	return 42;
}

/************************************************************************/
/*						Start of my code                                */
/************************************************************************/
// This is the constructor of a class that has been exported.
// see VRLib.h for the class definition
CVRLib::CVRLib()
{ 
	return; 
}

/************************************************************************/
/*						Init / Deinit Functions							*/
/************************************************************************/
//-----------------------------------------------------------------------------
// Name: Initialize
// Desc: Initializing routines for the library
//-----------------------------------------------------------------------------
BOOL CVRLib::Initialize()
{
	int		i;
		
	// Initializing storage arrays
	for(i=0; i<DATA_BYTE; i++)
	{
		m_uiSendData[i]			= (UINT) 0;
		m_cActualSendData[i]	= (CHAR) 0;
		m_cRetrievedData[i]	= (CHAR) 0;
	}

	// Initializing other variables
	m_dwBandMinFreq			= 0;
	m_dwBandMaxFreq			= 0;	
	m_bBandLoop			= FALSE;
	m_bTuneStatus		= FALSE;
	NumberOfBytesRead	= 0;
	DeviceDetected		= FALSE;
	DeviceHandle		= NULL;
	hEventObject		= NULL;
	hDevInfo			= NULL;
	ReadHandle			= NULL;

	// Set bytes data for sending to device
	SetPredefinedByte();
	// Default search field strength to most sensitive
	SetSearchFieldStrength(1);
	// Look for the Radio device
	DeviceDetected = FindTheHID();
	// Call important functions upon successful init
	if(DeviceDetected)
		SetTuneRange();
	
	return DeviceDetected;
}


//-----------------------------------------------------------------------------
// Name: Shutdown
// Desc: Perform tasks to switch off the radio and release unused resources
//-----------------------------------------------------------------------------
VOID CVRLib::Shutdown()
{
	if(DeviceDetected)
	{
		// Close the handles that are no longer in use
		CloseHandle(DeviceHandle);
		CloseHandle(ReadHandle);
	}
}


/************************************************************************/
/*							Tuner Functions								*/
/************************************************************************/
//-----------------------------------------------------------------------------
// Name: SetGPIOCTRL
// Desc: Takes in a GPIOCTRL and set the SDGPIOCTRL byte
//-----------------------------------------------------------------------------
HRESULT CVRLib::SetGPIOCTRL(GPIOCTRL control)
{
	// Refer to header file for more info and the tpyes of GPIOCTRL
	switch(control)
	{
	case GPLEDOFF:
		m_uiSendData[SDGPIOCTRL] = GPIO_LEDOFF;
		break;
	case GPLEDON:
		m_uiSendData[SDGPIOCTRL] = GPIO_LEDON;
		break;
	case GPREADFREQ:
		m_uiSendData[SDGPIOCTRL] = GPIO_READFREQ;
		break;
	case GPSENDDATA:
		m_uiSendData[SDGPIOCTRL] = GPIO_SENDDATA;
		break;
	case GPMUTE:
		m_uiSendData[SDGPIOCTRL] = GPIO_MUTE;
		break;
	case GPUNMUTE:
		m_uiSendData[SDGPIOCTRL] = GPIO_UNMUTE;
		break;
	case GPFIRMVER:
		m_uiSendData[SDGPIOCTRL] = GPIO_FIRMVER;
		break;
	case GPLEDRED:
		m_uiSendData[SDGPIOCTRL] = GPIO_LEDRED;
		break;
	case GPLEDGREEN:
		m_uiSendData[SDGPIOCTRL] = GPIO_LEDGREEN;
		break;
	case GPLEDRECSTATE:
		m_uiSendData[SDGPIOCTRL] = GPIO_LEDRECSTATE;
		break;
	}

	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: SetSearchFieldStrength
// Desc: Set the search field strength bits in Tuning Code Byte 2
//-----------------------------------------------------------------------------
HRESULT CVRLib::SetSearchFieldStrength(int strength)
{
	// Reset the the field strength bits to zero before setting it again.
	m_uiSendData[SDTCBYTE_2] &= 0xFC;	// 0xFC = 1111 1100
	
	switch(strength)
	{
	case 1:
		m_uiSendData[SDTCBYTE_2] |= SEARCHFIELDSTR1;	// Most sensitive
		break;
	case 2:
		m_uiSendData[SDTCBYTE_2] |= SEARCHFIELDSTR2;
		break;
	case 3:
		m_uiSendData[SDTCBYTE_2] |= SEARCHFIELDSTR3;
		break;
	case 4:
		m_uiSendData[SDTCBYTE_2] |= SEARCHFIELDSTR4;	// Least sensitive
		break;
	default:
		m_uiSendData[SDTCBYTE_2] |= SEARCHFIELDSTR1;
		break;
	}

	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: SetSearchStatus
// Desc: Takes in a SR_STATUS and set the bit in Tuning Code Byte 1
//-----------------------------------------------------------------------------
HRESULT CVRLib::SetSearchStatus(SR_STATUS SrStatus)
{
	if(SrStatus == START)
		// Setting the start/end bit in TC byte 1
		m_uiSendData[SDTCBYTE_1] |= STARTBIT;
	else if (SrStatus == END)
		// Clearing the start/end bit in TC byte 1
		m_uiSendData[SDTCBYTE_1] &= ENDBIT;

	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: SetSearchDirection
// Desc: Takes in a SR_DIRECTN and set the bit in Tuning Code Byte 2
//-----------------------------------------------------------------------------
HRESULT CVRLib::SetSearchDirection(SR_DIRECTN SrDrtn)
{
	if(SrDrtn == UP)
		// Setting the up/down bit in TC byte 2
		m_uiSendData[SDTCBYTE_2] |= UPBIT;
	else if (SrDrtn == DOWN)
		// Clearing the start/end bit in TC byte 2
		m_uiSendData[SDTCBYTE_2] &= DOWNBIT;

	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: SetAuFormat
// Desc: Takes in an AUDIO_FORMAT, set bit in Tuning Code Byte 2 and send 
//		 data to the HID device.
//-----------------------------------------------------------------------------
HRESULT CVRLib::SetAuFormat(AUDIO_FORMAT AuFormat)
{
	if(AuFormat == STEREO)
		// Clearing the stereo/mono bit in TC byte 2
		m_uiSendData[SDTCBYTE_2] &= STEREOBIT;
	else if(AuFormat == MONO)
			// Setting the stereo/mono bit in TC byte 2
			m_uiSendData[SDTCBYTE_2] |= MONOBIT;

	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: GetAuFormat
// Desc: Returns an Audio Format
//-----------------------------------------------------------------------------
AUDIO_FORMAT CVRLib::GetAuFormat()
{
	DWORD	theAuFormat;
	
	// See if there is any device connected. If not, find the HID device
	if(!DeviceDetected)
		DeviceDetected = FindTheHID();
	
	if(DeviceDetected)
	{
		// Read data from HID Device
		ReadReport();

		theAuFormat = m_cRetrievedData[RDTUNESTATE];

		if(theAuFormat == 0)
			return STEREO;
		else if(theAuFormat == 1)
			return MONO;
	}
	
	return MONO;
}


//-----------------------------------------------------------------------------
// Name: SetPredefinedByte
// Desc: Set Programmable ports and searching field bits. The defined values
//		 be modified in the header file.
//-----------------------------------------------------------------------------
VOID CVRLib::SetPredefinedByte()
{
	// Setting the packetID
	m_uiSendData[SDPACKETID] = PACKETID;	
	// Setting the combination bits for programmable ports
	m_uiSendData[SDTCBYTE_2] &= PPORTBIT;
	// Setting the FM band
	m_uiSendData[SDTCBYTE_2] &= FMBIT;
}


//-----------------------------------------------------------------------------
// Name: SetTuneRange
// Desc: This function was supposed to return the frequency range to calling app. 
//		 However the app that is interfaced with this dll do not need it. This 
//		 function's main purpose is to set the FM frequency range.
//-----------------------------------------------------------------------------
VOID CVRLib::SetTuneRange()
{
	// Update the band range
	m_dwBandMinFreq = (FMLOW);		// Convert Freq to 10kHz
	m_dwBandMaxFreq = (FMHIGH);		// Convert Freq to 10kHz
}


//-----------------------------------------------------------------------------
// Name: SetFreqInSendData
// Desc: Takes in a frequency in kHz and convert them to values recognized by
//		 device. Set the freq bytes in SendData array
//-----------------------------------------------------------------------------
HRESULT CVRLib::SetFreqInSendData(DWORD dwFreqHz)
{	
	UINT	FreqData[FREQ_BYTE];	// Arr to store frequency data byte
	UINT	tmpFreqData;			// Temp frequency data byte					
		
	// Convert the freq to hex byte
	FreqAndHex(&dwFreqHz, FREQTOHEX);
	// Extracting the first byte of the frequency byte
	tmpFreqData = dwFreqHz / (16 * 16);
	FreqData[0] = (UINT) tmpFreqData;
	// Extracting the second byte of the frequency byte
	tmpFreqData *=  (16 * 16);
	tmpFreqData = dwFreqHz - tmpFreqData;
	FreqData[1] = (UINT) tmpFreqData;
		
	// Transfer freqeuncy data to actual sent data
	for(int i=0; i < FREQ_BYTE; i++)
		m_uiSendData[SDTCBYTE_3  + i] = FreqData[i];
	
	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: SetTuneFreq
// Desc: Takes in a frequency in kHz, convert them to data bytes and send the
//		 data to the USB controller.
//-----------------------------------------------------------------------------
HRESULT CVRLib::SetTuneFreq(DWORD dwFreqHz)
{	
	HRESULT	Result;
	
	// Set the freq bytes in SendData array
	Result = SetFreqInSendData(dwFreqHz);
	if(Result != S_OK)
		return E_FAIL;
	// Set GPIOCTRL command
	SetGPIOCTRL(GPSENDDATA);
	// Write data to device
	Result = SendDataToDevice();
	if(Result != S_OK)
		return E_FAIL;

	return Result;
}


//-----------------------------------------------------------------------------
// Name: GetTuneFreq
// Desc: Returns the current tuned frequency.
//-----------------------------------------------------------------------------
DWORD CVRLib::GetTuneFreq()
{	
	DWORD	dwTheFreq, dwFreqData1, dwFreqData2;
	CHAR	cFreqData[2];
	TCHAR	tc[MAX_PATH], tc2[MAX_PATH];

	dwTheFreq = dwFreqData1 = dwFreqData2 = 0;
	
	// Issue a read freq command
	SetGPIOCTRL(GPREADFREQ);
	SendDataToDevice();
	Sleep(600);
	// Retrieve the data from hardware
	ReadReport();
	// Transfer data over
	for(int i =0; i < FREQ_BYTE; i++)
		cFreqData[i] = m_cRetrievedData[RDTCBYTE_3 + i];

	// Process first byte
	wsprintf(tc, "%X", cFreqData[0]);
	StrtoHex(tc, &dwFreqData1);
	//wsprintf(test, "First Byte: %X\n", Byte1);
	//OutputDebugString(test);
	//wsprintf(test, "First Byte: %X\n", FreqByte1);
	//OutputDebugString(test);
	
	// Process second byte
	wsprintf(tc2, "%X", cFreqData[1]);
	StrtoHex(tc2, &dwFreqData2);
	//wsprintf(test, "Second Byte: %X\n", Byte2);
	//OutputDebugString(test);
	/*
	Background info:
	The data we retrieved from the CHAR array is normally okay except that when the first
	character is of value 8 and greater, there would be six F's preceeding the data. We have
	to remove this or else when we combine with the first byte, the result would not be
	accurate. There is no error checking for the first byte because the first character
	of the first byte would not reach 8 due to the range of the frequency
	*/	
	// Error processing 
	if(strlen(tc2) > 2)
		dwFreqData2 -= 0xFFFFFF00;
	//wsprintf(test, "1: %X\n", FreqByte1);
	//OutputDebugString(test);
	//wsprintf(test, "2: %X\n", FreqByte2);
	//OutputDebugString(test);
		
	// Shift the hexadecimal value to the left by two characters
	dwFreqData1 *= (16*16);
	// Combine the frequency bytes	
	dwTheFreq = dwFreqData1 + dwFreqData2;
	//wsprintf(test, "ActualFreq: %X\n", ActualFreq);
	//OutputDebugString(test);
	// Convert the bytes to freq
	FreqAndHex(&dwTheFreq, HEXTOFREQ);

	return dwTheFreq;
}


//-----------------------------------------------------------------------------
// Name: StartSearch
// Desc: Start the searching algorithm in the direction stated and return the
//		 tuned frequency
//-----------------------------------------------------------------------------
DWORD CVRLib::StartSearch(SR_DIRECTN SrDrtn, DWORD dwCurFreq)
{
	DWORD	ActualFreq, lastDigit;
	LONG	testFreq;
	//CHAR	test[MAX_PATH];
		
	if(SrDrtn == UP)
		// Set an offset of 100 kHz before searching
		dwCurFreq += 10;
	else if(SrDrtn == DOWN)
			// Set an offset of 100 kHz before searching
			dwCurFreq -= 10;
		
	// Tune to the offset freq
	SetTuneFreq(dwCurFreq);
	//wsprintf(test, "%d\n", dwCurFreq);
	//OutputDebugString(test);
		
	// Start a search, with the stated direction
	SetSearchStatus(START);
	SetSearchDirection(SrDrtn);
	SetGPIOCTRL(GPSENDDATA);
	SendDataToDevice();
	
	Sleep(55);
		
	// Keep polling the radio chip to see if dataline has gone low
	do
	{
		ReadReport();
		//wsprintf(test, "Compare: %X", m_cRetrievedData[RDTCBYTE_1]);
		//OutputDebugString(test);
		if(m_cRetrievedData[RDTCBYTE_1] == (CHAR) 0)
		{
			//Data line has gone low
			m_bTuneStatus = TRUE;
			break;
		}
	} while(!m_bTuneStatus);

	// From this point onwards, any command that we send, should indicate search has ended
	SetSearchStatus(END);

	// Retrieved the tuned frequency
	ActualFreq = GetTuneFreq();
	
	// Test to see if frequency bytes return 0 value
	testFreq = ActualFreq;
	if(testFreq < 0)
	{
		//OutputDebugString("Corrective Algorithm 1");
		// If yes, it means that the tuner could not retrieve any frequency
		// We will set the offseted frequency 
		SetTuneFreq(dwCurFreq);
		return dwCurFreq;
	}
	
	// If received bytes are non zero, pass the freq to the Radio's Freq variable
	dwCurFreq = ActualFreq;
	//wsprintf(test, "RadioFreq: %d\n", dwCurFreq);
	//OutputDebugString(test);
	
	// Test to see if return frequency is out of range
	if(ActualFreq <= m_dwBandMinFreq || ActualFreq >= m_dwBandMaxFreq)
	{
		//OutputDebugString("Corrective Algorithm 2");
		// Set frequency to the min or max value of the frequency range
		if(SrDrtn == UP)
			dwCurFreq = m_dwBandMaxFreq;
		if(SrDrtn == DOWN)
			dwCurFreq = m_dwBandMinFreq;
	
		//m_bBandLoop = TRUE;
		//StartSearch(SrDrtn, dwCurFreq);
		SetTuneFreq(dwCurFreq);
		return dwCurFreq;
	}

	// End of search algorithm. Reset this variable
	//m_bBandLoop = FALSE;
	// Algorithm to round off value
	// Get the last digit of the frequency
	lastDigit = dwCurFreq % 10;
	// Round off the frequency to the nearest 50 kH. 
	if(lastDigit < 3)
		dwCurFreq -= lastDigit;
	else if(lastDigit < 8)
			dwCurFreq = dwCurFreq - lastDigit + 5;
		else
			dwCurFreq = dwCurFreq - lastDigit + 10;
	// Set the radio to the current frequency
	//wsprintf(test, "RadioFreq: %d\n", dwCurFreq);
	//OutputDebugString(test);
	SetTuneFreq(dwCurFreq);
				
	return dwCurFreq;
}


//-----------------------------------------------------------------------------
// Name: MuteVolume
// Desc: Hardware Mute
//-----------------------------------------------------------------------------
HRESULT CVRLib::MuteVolume()
{
	HRESULT Result;
	
	// Set GPIOCTRL command and send to device
	SetGPIOCTRL(GPMUTE);
	Result = SendDataToDevice();
	
	return Result;
}


//-----------------------------------------------------------------------------
// Name: UnMuteVolume
// Desc: Hardware UnMute
//-----------------------------------------------------------------------------
HRESULT CVRLib::UnMuteVolume()
{
	HRESULT Result;
	
	// Set GPIOCTRL command and send to device
	SetGPIOCTRL(GPUNMUTE);
	Result = SendDataToDevice();
	
	return Result;
}


//-----------------------------------------------------------------------------
// Name: SetLEDRed
// Desc: Set LED light to red color
//-----------------------------------------------------------------------------
HRESULT CVRLib::SetLEDRed()
{
	HRESULT Result;
	
	SetGPIOCTRL(GPLEDRED);
	Result = SendDataToDevice();
	Sleep(100);
	
	return Result;
}


//-----------------------------------------------------------------------------
// Name: SetLEDGreen
// Desc: Set LED light to green color
//-----------------------------------------------------------------------------
HRESULT CVRLib::SetLEDGreen()
{
	HRESULT Result;
	
	SetGPIOCTRL(GPLEDGREEN);
	Result = SendDataToDevice();
	Sleep(100);
	
	return Result;
}


//-----------------------------------------------------------------------------
// Name: SetLEDRecState
// Desc: Set LED light to blinking green color
//-----------------------------------------------------------------------------
HRESULT CVRLib::SetLEDRecState()
{
	HRESULT Result;
	
	SetGPIOCTRL(GPLEDRECSTATE);
	Result = SendDataToDevice();
	Sleep(100);
	
	return Result;
}


//-----------------------------------------------------------------------------
// Name: GetIRCode
// Desc: Return the last received IR code
//-----------------------------------------------------------------------------
DWORD CVRLib::GetIRCode()
{
	DWORD	IRCode;

	// See if there is any device connected. If not, find the HID device
	DeviceDetected = FindTheHID();
	
	if(DeviceDetected)
	{
		// Read data from HID device
		ReadReport();

		// Retrieve last IR code received
		IRCode = m_cRetrievedData[RDIRCODE];

		return IRCode;
	}

	return 0xFF;
}


//-----------------------------------------------------------------------------
// Name: GetFirmwareVer
// Desc: Returns the Firmware Version
//-----------------------------------------------------------------------------
HRESULT CVRLib::GetFirmwareVer(DWORD* dwFmVer1, DWORD* dwFmVer2)
{
	CHAR	cByte1, cByte2;
	TCHAR	display[MAX_PATH];
	DWORD	dwByte1, dwByte2;
		
	// See if there is any device connected. If not, find the HID device
	if(!DeviceDetected)
		DeviceDetected = FindTheHID();
	
	if(DeviceDetected)
	{
		// Send a get firmware version command to device
		SetGPIOCTRL(GPFIRMVER);
		SendDataToDevice();
		Sleep(600);

		// Read data from HID device
		ReadReport();

		cByte1 = m_cRetrievedData[RDTCBYTE_1];
		cByte2 = m_cRetrievedData[RDTCBYTE_2];

		wsprintf(display, "%X", cByte1);
		StrtoHex(display, &dwByte1);
	
		wsprintf(display, "%X", cByte2);
		StrtoHex(display, &dwByte2);
		if(strlen(display) > 2)
			dwByte2 -= 0xFFFFFF00;
		
		*dwFmVer1 = dwByte1;
		*dwFmVer2 = dwByte2;
	
		return S_OK;
	}

	*dwFmVer1 = 0;
	*dwFmVer2 = 0;
	return E_FAIL;
}


//-----------------------------------------------------------------------------
// Name: GetDLLVer
// Desc: Returns the DLL Version
//-----------------------------------------------------------------------------
HRESULT CVRLib::GetDLLVer(INT* iDllVer1, INT* iDllVer2, INT* iDllVer3)
{
	*iDllVer1 = DLL_VERMAJ;
	*iDllVer2 = DLL_VERMED;
	*iDllVer3 = DLL_VERMIN;

	return S_OK;
}


/************************************************************************/
/*						Calculation Functions							*/
/************************************************************************/
//-----------------------------------------------------------------------------
// Name: StrtoHex
// Desc: This function helps to convert a text string to hexadecimal values
//-----------------------------------------------------------------------------
VOID CVRLib::StrtoHex(char *inStr, DWORD *dwData) 
{
	DWORD	dwtmp;	
	char	ctmp;
	
	*dwData = 0;
	while(1) 
	{
		// Storing a single char and increment string pointer to next char
		ctmp = *inStr++;
		// Exit if reached the end of the string
		if (ctmp==NULL) break;
		// Shift the previous hex value to the left (i.e. AB -> AB0)
		*dwData *= 16;
		
		dwtmp=0;
		// Deocde char and convert to hex value
		if ( (ctmp>='0')&&(ctmp<='9') )
			dwtmp = (int) (ctmp - ((char)'0')) ;			
		else if ( (ctmp>='a')&&(ctmp<='f') )
			dwtmp = (int)(ctmp - 'a') + 10;			
		else if ( (ctmp>='A')&&(ctmp<='F') )
			dwtmp = (int)(ctmp - 'A') + 10;			
		else 
			// If char is not of hex convention, ignore char and 
			// shift hex value to the right (i.e. AB0 -> AB)
			*dwData /= 16;	
		
		// Add hex value together (i.e. AB0 + 9 -> AB9)
		*dwData += dwtmp;
	}
}


//-----------------------------------------------------------------------------
// Name: FreqAndHex
// Desc: This function helps to convert frequency values to hexadeciaml values
//		 and vice versa, depending on the 'choice', FREQTOHEX and HEXTOFREQ.
//-----------------------------------------------------------------------------
VOID CVRLib::FreqAndHex(DWORD* theFreq, int choice)
{
	switch(choice) 
	{
	case FREQTOHEX:
		// Add the intermedia frequency, 10.7 Mhz. Our frequency is in 10kHz
		*theFreq += 1070;
		// Convert frequency to Hz
		*theFreq *= 10000;
		
		// Convert frequency to values according to table
		*theFreq /= 12500;
		break;
	case HEXTOFREQ:
		// Convert frequency to values according to table
		*theFreq *= 12500;
		
		// Convert frequency to 10kHz
		*theFreq /= 10000;
		// Subtract the intermedia frequency to get correct frequency value
		*theFreq -= 1070;
		break;
	}
}


/************************************************************************/
/*						USB Communication Functions						*/
/************************************************************************/
//-----------------------------------------------------------------------------
// Name: FindTheHID
// Desc: This function uses a series of API calls to find a HID with a matching
//		 Vendor and Product ID
//-----------------------------------------------------------------------------
BOOL CVRLib::FindTheHID()
{
	HIDD_ATTRIBUTES				Attributes;
	SP_DEVICE_INTERFACE_DATA	devInfoData;		// Structure that contains device interface data
	BOOL						LastDevice;			// TRUE if no more device detected
	BOOL						MyDeviceDetected;	// TRUE if our HID device found
	LONG						Result;
	ULONG						Length, Required;
	int							MemberIndex;		// Variable to keep track of number of devices
		
	//These are the vendor and product IDs to look for.
	//Uses Celestix Network's Vendor ID.
	//const unsigned int VendorID = 0x0cb6;	//Kiew
	//const unsigned int ProductID = 0x0001;

	const unsigned int VendorID = RADIATOR_VID;	//Kiew
	const unsigned int ProductID = RADIATOR_PID;

	// Pre-operation initialization
	Length	= Required = 0;
	detailData		= NULL;
	DeviceHandle	= NULL;
	
	/*
	API function: HidD_GetHidGuid
	Get the GUID for all system HIDs.
	Returns: the GUID in HidGuid.
	*/

	HidD_GetHidGuid(&HidGuid);	
	
	/*
	API function: SetupDiGetClassDevs
	Returns: a handle to a device information set for all installed devices.
	Requires: the GUID returned by GetHidGuid.
	*/
	
	hDevInfo=SetupDiGetClassDevs 
		(&HidGuid, 
		NULL, 
		NULL, 
		DIGCF_PRESENT|DIGCF_DEVICEINTERFACE);
		
	devInfoData.cbSize = sizeof(devInfoData);

	//Step through the availabzle devices looking for the one we want. 
	//Quit on detecting the desired device or checking all available devices without success.
	MemberIndex = 0;
	LastDevice = FALSE;

	do
	{
		MyDeviceDetected=FALSE;
		
		/*
		API function: SetupDiEnumDeviceInterfaces
		On return, MyDeviceInterfaceData contains the handle to a
		SP_DEVICE_INTERFACE_DATA structure for a detected device.
		Requires:
		The DeviceInfoSet returned in SetupDiGetClassDevs.
		The HidGuid returned in GetHidGuid.
		An index to specify a device.
		*/

		Result=SetupDiEnumDeviceInterfaces 
			(hDevInfo, 
			0, 
			&HidGuid, 
			MemberIndex, 
			&devInfoData);

		if (Result != 0)
		{
			//A device has been detected, so get more information about it.

			/*
			API function: SetupDiGetDeviceInterfaceDetail
			Returns: an SP_DEVICE_INTERFACE_DETAIL_DATA structure
			containing information about a device.
			To retrieve the information, call this function twice.
			The first time returns the size of the structure in Length.
			The second time returns a pointer to the data in DeviceInfoSet.
			Requires:
			A DeviceInfoSet returned by SetupDiGetClassDevs
			The SP_DEVICE_INTERFACE_DATA structure returned by SetupDiEnumDeviceInterfaces.
			
			The final parameter is an optional pointer to an SP_DEV_INFO_DATA structure.
			This application doesn't retrieve or use the structure.			
			If retrieving the structure, set 
			MyDeviceInfoData.cbSize = length of MyDeviceInfoData.
			and pass the structure's address.
			*/
			
			//Get the Length value.
			//The call will return with a "buffer too small" error which can be ignored.
			Result = SetupDiGetDeviceInterfaceDetail 
				(hDevInfo, 
				&devInfoData, 
				NULL, 
				0, 
				&Length, 
				NULL);

			//Allocate memory for the hDevInfo structure, using the returned Length.
			detailData = (PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(Length);
			
			//Set cbSize in the detailData structure.
			detailData -> cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

			//Call the function again, this time passing it the returned buffer size.
			Result = SetupDiGetDeviceInterfaceDetail 
				(hDevInfo, 
				&devInfoData, 
				detailData, 
				Length, 
				&Required, 
				NULL);

			//Open a handle to the device.

			/*
			API function: CreateFile
			Returns: a handle that enables reading and writing to the device.
			Requires:
			The DevicePath in the detailData structure
			returned by SetupDiGetDeviceInterfaceDetail.
			*/

			DeviceHandle=CreateFile 
				(detailData->DevicePath, 
				GENERIC_READ|GENERIC_WRITE, 
				FILE_SHARE_READ|FILE_SHARE_WRITE, 
				NULL,
				OPEN_EXISTING, 
				0, 
				NULL);

			if(DeviceHandle == INVALID_HANDLE_VALUE)
				OutputDebugString("CreateFile (FindTheHID)\n");

			/*
			API function: HidD_GetAttributes
			Requests information from the device.
			Requires: the handle returned by CreateFile.
			Returns: a HIDD_ATTRIBUTES structure containing
			the Vendor ID, Product ID, and Product Version Number.
			Use this information to decide if the detected device is
			the one we're looking for.
			*/

			//Set the Size to the number of bytes in the structure.
			Attributes.Size = sizeof(Attributes);

			Result = HidD_GetAttributes 
				(DeviceHandle, 
				&Attributes);
			
			if(Result == FALSE)
				OutputDebugString("HidD_GetAttributes\n");
			
			//Is it the desired device?
			MyDeviceDetected = FALSE;
			
			if (Attributes.VendorID == VendorID)
			{
				if (Attributes.ProductID == ProductID)
				{
					//Both the Product and Vendor IDs match.
					MyDeviceDetected = TRUE;
					OutputDebugString("Device detected\n");
					//Get the device's capablities.
					GetDeviceCapabilities();
					PrepareForOverlappedTransfer();
				} //if (Attributes.ProductID == ProductID)

				else
					//The Product ID doesn't match.
					CloseHandle(DeviceHandle);
			} //if (Attributes.VendorID == VendorID)

			else
				//The Vendor ID doesn't match.
				CloseHandle(DeviceHandle);

		//Free the memory used by the detailData structure (no longer needed).
		free(detailData);
		}  //if (Result != 0)

		else
			//SetupDiEnumDeviceInterfaces returned 0, so there are no more devices to check.
			LastDevice=TRUE;

		//If we haven't found the device yet, and haven't tried every available device,
		//try the next one.
		MemberIndex++;
	} //do
	while ((LastDevice == FALSE) && (MyDeviceDetected == FALSE));

	if (MyDeviceDetected == FALSE)
		OutputDebugString("Device not detected\n");
	else
		OutputDebugString("Device detected\n");

	//Free the memory reserved for hDevInfo by SetupDiClassDevs.
	SetupDiDestroyDeviceInfoList(hDevInfo);
		
	return MyDeviceDetected;
}

//-----------------------------------------------------------------------------
// Name: GetDeviceCapabilities
// Desc: Get the Capabilities structure for the device.
//-----------------------------------------------------------------------------
VOID CVRLib::GetDeviceCapabilities()
{
	PHIDP_PREPARSED_DATA	PreparsedData;
	LONG					Result;
		
	/*
	API function: HidD_GetPreparsedData
	Returns: a pointer to a buffer containing the information about the device's capabilities.
	Requires: A handle returned by CreateFile.
	There's no need to access the buffer directly,
	but HidP_GetCaps and other API functions require a pointer to the buffer.
	*/
	
	Result = HidD_GetPreparsedData(DeviceHandle, &PreparsedData);
	if(Result == FALSE)
		OutputDebugString("HidD_GetPreparsedData\n");
	
	/*
	API function: HidP_GetCaps
	Learn the device's capabilities.
	For standard devices such as joysticks, you can find out the specific
	capabilities of the device.
	For a custom device, the software will probably know what the device is capable of,
	and the call only verifies the information.
	Requires: the pointer to the buffer returned by HidD_GetPreparsedData.
	Returns: a Capabilities structure containing the information.
	*/
	
	Result = HidP_GetCaps(PreparsedData, &Capabilities);
	if(Result == HIDP_STATUS_INVALID_PREPARSED_DATA)
		OutputDebugString("HidP_GetCaps\n");
		
	//No need for PreparsedData any more, so free the memory it's using.
	Result = HidD_FreePreparsedData(PreparsedData);
	if(Result == FALSE)
		OutputDebugString("HidD_FreePreparsedData\n") ;
}


//-----------------------------------------------------------------------------
// Name: PrepareForOverlappedTransfer
// Desc: To get a handle that can perform overlapped data transfer. This handle
//		 is going to be used by ReadFile()
//-----------------------------------------------------------------------------
VOID CVRLib::PrepareForOverlappedTransfer()
{
	// 03_04_03_Mike : To solve 2 problems: 
	//                 1) memory leak issue in Win2K's hidclass.sys
	//                 2) Slow reponse in WinME
//	Get another handle to the device for the overlapped ReadFiles.
//	m_hReadHandle=CreateFile 
//		(m_DetailData->DevicePath, 
//		GENERIC_READ|GENERIC_WRITE, 
//		FILE_SHARE_READ|FILE_SHARE_WRITE,
//		NULL, 
//		OPEN_EXISTING, 
//		FILE_FLAG_OVERLAPPED, 
//		NULL);
	
	if(ReadHandle == INVALID_HANDLE_VALUE)
		OutputDebugString("CreateFile (PrepareForOverlappedTransfer)\n");
	
	//Get an event object for the overlapped structure.
	
	/*API function: CreateEvent
	Requires:
	Security attributes or Null
	Manual reset (true). Use ResetEvent to set the event object's state to non-signaled.
	Initial state (true = signaled) 
	Event object name (optional)
	Returns: a handle to the event object
	*/
	
	if (hEventObject == 0)
	{
		hEventObject = CreateEvent 
			(NULL, 
			TRUE, 
			TRUE, 
			"");
		if(hEventObject == NULL)
			OutputDebugString("CreateEvent\n") ;
		
		//Set the members of the overlapped structure.
		HIDOverlapped.hEvent = hEventObject;
		HIDOverlapped.Offset = 0;
		HIDOverlapped.OffsetHigh = 0;
	}
}


//-----------------------------------------------------------------------------
// Name: ReadReport
// Desc: Main function for reading data from radio chip. Retreived data goes
//		 to a CHAR array. ReadFile function is using overlapped IO.
//-----------------------------------------------------------------------------
HRESULT CVRLib::ReadReport()
{
	DWORD	Result, dwByte;
	TCHAR	tcByte[9];
		
	// 03_04_03_Mike : To solve 2 problems: 
	//                 1) memory leak issue in Win2K's hidclass.sys
	//                 2) Slow reponse in WinME
	// 
	
	if(!ReadHandle)
		ReadHandle=CreateFile 
		(detailData->DevicePath, 
		GENERIC_READ|GENERIC_WRITE, 
		FILE_SHARE_READ|FILE_SHARE_WRITE,
		NULL, 
		OPEN_EXISTING, 
		FILE_FLAG_OVERLAPPED, 
		NULL);
	
	//Read a report from the device.
	
	/*API call:ReadFile
	'Returns: the report in InputReport.
	'Requires: a device handle returned by CreateFile
	'(for overlapped I/O, CreateFile must be called with FILE_FLAG_OVERLAPPED),
	'the Input report length in bytes returned by HidP_GetCaps,
	'and an overlapped structure whose hEvent member is set to an event object.
	*/
	
	Result = ReadFile
		(ReadHandle, 
		m_cRetrievedData, 
		Capabilities.InputReportByteLength,
		&NumberOfBytesRead,
		(LPOVERLAPPED) &HIDOverlapped); 
	
	if(Result == 0)
	{
		Result = GetLastError();
		if(Result != ERROR_IO_PENDING)
			OutputDebugString("ReadFile\n") ;
	}
	
	/*API call:WaitForSingleObject
	'Used with overlapped ReadFile.
	'Returns when ReadFile has received the requested amount of data or on timeout.
	'Requires an event object created with CreateEvent
	'and a timeout value in milliseconds.
	*/
	
	Result = WaitForSingleObject 
		(hEventObject, 
		4000);
		
	switch (Result)
	{
	case WAIT_OBJECT_0:
		{
			Result = S_OK;
			break;
		}
	case WAIT_TIMEOUT:
		{
			OutputDebugString("ReadFile timeout\n");
			//Cancel the Read operation.
			
			/*API call: CancelIo
			Cancels the ReadFile
			Requires the device handle.
			Returns non-zero on success.
			*/
			
			Result = CancelIo(ReadHandle);
			
			//A timeout may mean that the device has been removed. 
			//Close the device handles and set DeviceDetected = False 
			//so the next access attempt will search for the device.
			CloseHandle(ReadHandle);
			CloseHandle(DeviceHandle);
			OutputDebugString("Can't read from device\n");
			DeviceDetected = FALSE;
			ResetEvent(hEventObject);
			return E_FAIL;
			break;
	default:
		OutputDebugString("Undefined error\n");
		break;
		}
	}
	
	// 03_04_03_Mike : To solve 2 problems: 
	//                 1) memory leak issue in Win2K's hidclass.sys
	//                 2) Slow reponse in WinME
	CloseHandle(ReadHandle);
	ReadHandle = NULL;	
	
	/*
	API call: ResetEvent
	Sets the event object to non-signaled.
	Requires a handle to the event object.
	Returns non-zero on success.
	*/
	
	// We have to reset the event to an unsignalled state. We need to do this 
	// manually because hEventObject is created in such a way.
	ResetEvent(hEventObject);

	// Checks the PacketID. If matched, retained data. If mismatched, erase data
	wsprintf(tcByte, "%X", m_cRetrievedData[RDPACKETID]);
	StrtoHex(tcByte, &dwByte);
	
	// If the value of the byte is >= 80, then it would be padded with Fs for up to
	// eight chars. which is why we need to subtract the addition value to get the
	// correct reading.
	if(strlen(tcByte) > 2)
		dwByte -= 0xFFFFFF00;

	if(dwByte - 0xB1 != 0)
	{
		for(int i=1; i<DATA_BYTE; i++)
			m_cRetrievedData[i] = (char) 0;
	}
	
	return Result;
}


//-----------------------------------------------------------------------------
// Name: WriteReport
// Desc: Main function for writing data to the radio chip. Send data to radio
//		 chip that are stored in  an output array. 
//-----------------------------------------------------------------------------
HRESULT CVRLib::WriteReport()
{
	//Send a report to the device.
	DWORD	BytesWritten = 0;
	INT		Index = 0;
	ULONG	Result;
	//TCHAR	outputString[MAX_PATH];
		
	/*
	API Function: WriteFile
	Sends a report to the device.
	Returns: success or failure.
	Requires:
	The device handle returned by CreateFile.
	The Output Report length returned by HidP_GetCaps,
	A report to send.
	*/
	
	Result = WriteFile 
		(DeviceHandle, 
		m_cActualSendData, 
		Capabilities.OutputReportByteLength, 
		&BytesWritten, 
		NULL);

	if (Result != 0)
	{
		//The WriteFile failed, so close the handle, display a message,
		//and set DeviceDetected to FALSE so the next attempt will look for the device.
		OutputDebugString("Can't write to device\n");
		//wsprintf(outputString, "Bytes written: %d\n", BytesWritten);
		//OutputDebugString(outputString);
		CloseHandle(DeviceHandle);
		CloseHandle(ReadHandle);
		DeviceDetected = FALSE;
		return E_FAIL;
	}
	
	//Display the result of the API call and the report bytes.
	//OutputDebugString("WriteFile succeed\n");
	//wsprintf(outputString, "Bytes written: %d\n", BytesWritten);
	//OutputDebugString(outputString);
	return S_OK;
}


//-----------------------------------------------------------------------------
// Name: SendDataToDevice
// Desc: This function will prepare the AcutalSendData buffer before sending to
//		 the radio device
//-----------------------------------------------------------------------------
HRESULT	CVRLib::SendDataToDevice()
{
	HRESULT	Result;
	//CHAR	test[MAX_PATH];
	
	// See if there is any device connected. If not, find the HID device
	if(!DeviceDetected)
	{
		DeviceDetected = FindTheHID();
		if(!DeviceDetected)
			return E_FAIL;
	}
	
	// Converting data from UINT to CHAR
	for(int i=1; i<DATA_BYTE; i++)
	{
		m_cActualSendData[i] = (char) m_uiSendData[i];
		//wsprintf(test, "%X\n", m_cActualSendData[i]);
		//OutputDebugString(test);
	}
	
	Result = WriteReport();
	
	return Result;
}


/************************************************************************/
/*							Export Functions							*/
/************************************************************************/
VRLIB_API BOOL Initialize()
{
	pLib = NULL;
	pLib = new CVRLib();
	return pLib->Initialize();
}

VRLIB_API VOID Shutdown()
{
	pLib->Shutdown();
	
	if(pLib)
	{
		pLib = NULL;
		delete pLib;
	}
}

VRLIB_API HRESULT SetAuFormat(AUDIO_FORMAT AuFormat)
{
	return pLib->SetAuFormat(AuFormat);
}

VRLIB_API AUDIO_FORMAT GetAuFormat()
{
	return pLib->GetAuFormat();
}

VRLIB_API HRESULT SetTuneFreq(DWORD dwFreq)
{
	return pLib->SetTuneFreq(dwFreq);
}

VRLIB_API DWORD StartSearch(SR_DIRECTN SrDrtn, DWORD dwCurFreq)
{
	return pLib->StartSearch(SrDrtn, dwCurFreq);
}

VRLIB_API HRESULT MuteVolume()
{
	return pLib->MuteVolume();
}

VRLIB_API HRESULT UnMuteVolume()
{
	return	pLib->UnMuteVolume();
}

VRLIB_API HRESULT SetLEDRecStart()
{
	return	pLib->SetLEDRecState();
}

VRLIB_API HRESULT SetLEDRecStop()
{
	return	pLib->SetLEDGreen();
}

VRLIB_API HRESULT SetLEDAppStart()
{
	return	pLib->SetLEDGreen();
}

VRLIB_API HRESULT SetLEDAppStop()
{
	return	pLib->SetLEDRed();
}

VRLIB_API HRESULT GetFirmwareVer(DWORD* dwFmVer1, DWORD* dwFmVer2)
{
	return pLib->GetFirmwareVer(dwFmVer1, dwFmVer2);
}

VRLIB_API HRESULT GetDLLVer(INT* iDllVer1, INT* iDllVer2, INT* iDllVer3)
{
	return pLib->GetDLLVer(iDllVer1, iDllVer2, iDllVer3);
}
