// VidzRadioProdTestDlg.h : header file
//

#if !defined(AFX_VIDZRADIOPRODTESTDLG_H__F6D4C9FB_FC8C_4BFC_9663_66D509FACC36__INCLUDED_)
#define AFX_VIDZRADIOPRODTESTDLG_H__F6D4C9FB_FC8C_4BFC_9663_66D509FACC36__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <mmsystem.h>
#include "VRLib.h"
#include "SettingsDlg.h"
#include "ColorStaticST.h"

#define SOFTWARE_VERSION 100

typedef enum
{
	LEFT		= 0,
	RIGHT		= 1,
	CENTER		= 2,
} BALANCE;

/////////////////////////////////////////////////////////////////////////////
// CVidzRadioProdTestDlg dialog

class CVidzRadioProdTestDlg : public CDialog
{
// Construction
public:
	CVidzRadioProdTestDlg(CWnd* pParent = NULL );	// standard constructor
		
	// Helper functions
	void SetTestButtons(BOOL bState);				// True to enable, FALSE to disable test buttons
	void DisplayRCMessage(UINT uiChoice);			// Display preset remote control message
	void ReadWriteTestData(BOOL bReadStatus);		// True to read test data, FALSE to write
	void UpdateTestData();							// Update display for test data
	void PlayTestResultSound(INT iTest);			// Play the respectively sound for test result
	BOOL ProbeAllTestStates();						// Returns TRUE if all tests are done
	// Mixer Functions
	VOID CreateMixerObj();								// Creates the mixer object for line in volume control
	VOID SetMixerBal(BALANCE bal);						// Set the mixer balance. LEFT, RIGHT or CENTER
	VOID GetMixerBal(DWORD* dwLeft, DWORD* dwRight);	// Return the volume value for left and right channel

	
	// Class variables
	CVRLib*			m_pLib;				// Pointer to VRLib.dll
	CSettingsDlg	m_dlgSettings;		// The main settings dialog object
	// Variables to store info of test
	UINT		m_uiPresetFreq;			// The preset frequency for the tune test
	UINT		m_uiSenStaFreq1;		// The preset frequency for the sensitivity test
	UINT		m_uiSenStaFreq2;		// The preset frequency for the sensitivity test
	UINT		m_uiSenStaFreq3;		// The preset frequency for the sensitivity test
	UINT		m_uiSrFrequency;		// The starting frequency for the search test
	UINT		m_uiSrDirection;		// The direction to search for the search test
	UINT		m_uiRemoteChoice;		// The remote control button to test for
	UINT		m_uiCurFirmVer;			// The current firmware version of the hardware
	BOOL		m_bDataLoaded;			// Returns true if all data is loaded
	DWORD		m_dwBtnCode[32];		// Array to store the IR code for the remote control buttons
	BOOL		m_bAutoTest;			// Returns true if automatic test is enabled
	BOOL		m_bEnableSound;			// Returns true if sound is enabled
	BOOL		m_bEnableRC;			// Returns true if remote control is enabled
	INT			m_iATCount;				// Counter to keep track of auto test status
	BOOL		m_bAutoTestStarted;		// Returns true if auto test has started
	BOOL		m_bTestStarted;			// Returns true if test has started
	// Variable to keep track of search test info
	INT			m_iSearchCount;			// Counter to kep track of the number of times search is performed
	DWORD		m_dwNewSearchFreq;		// Variable to store the new frequency after search.
										// It is required to implement continuous search in desired direction.
	// Bool variables to keep track of test states for auto test
	BOOL		m_bTuneTest;
	BOOL		m_bSTPinTest;
	BOOL		m_bSenTest;
	BOOL		m_bSearchTest;
	BOOL		m_bAuLeftTest;
	BOOL		m_bAuRightTest;
	BOOL		m_bAuBothTest;
	BOOL		m_bLEDGreen;
	BOOL		m_bLEDRed;
	BOOL		m_bRCTest;
	BOOL		m_bAllTestDone;
	// Variable to prevent over sensitive remote response
	DWORD		m_dwLastIRCode;			// Stores last received IR code
	// Mixer variables
	MMRESULT	m_mmr;					// Variable to store Mixer functions results
	HMIXER		m_hmixer;				// A handle to an opened mixer device
	MIXERCONTROLDETAILS				m_mxCtrlDl;			// Variable to manipulate volume
	MIXERCONTROLDETAILS_UNSIGNED	m_mxCtrlDlUS[2];	// Variable to manipulate volume
	
// Dialog Data
	//{{AFX_DATA(CVidzRadioProdTestDlg)
	enum { IDD = IDD_VIDZRADIOPRODTEST_DIALOG };
	CColorStaticST	m_stcSTPinRS;
	CColorStaticST	m_stcSenSta1RS;
	CColorStaticST	m_stcSenSta2RS;
	CColorStaticST	m_stcSenSta3RS;
	CColorStaticST	m_stcTuneTestRS;
	CStatic			m_csDevice;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CVidzRadioProdTestDlg)
	public:
	virtual BOOL PreTranslateMessage(MSG* pMsg);
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	HICON m_hIcon;

	// Generated message map functions
	//{{AFX_MSG(CVidzRadioProdTestDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	afx_msg void OnInitialize();
	afx_msg void OnSettingsConfig();
	afx_msg void OnHelpAbout();
	afx_msg void OnQuit();
	afx_msg void OnTuneTest();
	afx_msg void OnStereoPinTest();
	afx_msg void OnSensitivityTest();
	afx_msg void OnSearchTest();
	afx_msg void OnAudioLeft();
	afx_msg void OnAudioRight();
	afx_msg void OnAudioLeftAndRight();
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnClose();
	afx_msg void OnRcNtDetected();
	afx_msg void OnLedGreen();
	afx_msg void OnLedRed();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_VIDZRADIOPRODTESTDLG_H__F6D4C9FB_FC8C_4BFC_9663_66D509FACC36__INCLUDED_)
